import imp
import sys
import types
import importlib

__all__ = ['reload_module', 'reload_all']


"""
Python 3.4+ compatible is_builtin function.
"""
try:
    _is_builtin = imp.is_builtin
except AttributeError:
    def _is_builtin(module_name):
        return module_name in sys.builtin_module_names


"""
Python 2/3/3.4+ compatible reload function.
"""
try:
    _reload = reload
except NameError:
    try:
        _reload = importlib.reload
    except AttributeError:
        _reload = imp.reload


def _get_dependencies(module):
    """
    Find modules imported by the given module.
    """
    for variable in vars(module).values():
        if not isinstance(variable, types.ModuleType):
            continue
        if _is_builtin(variable.__name__):
            continue
        yield variable


def _get_modules_in_order(module, done=set()):
    """
    Get imported modules in depth first order.
    """
    done |= {module.__name__}
    for dependency in _get_dependencies(module):
        if dependency.__name__ not in done:
            for subdependency in _get_modules_in_order(dependency, done):
                yield subdependency
    yield module


def reload_module(module):
    """
    Reload module and its dependencies.
    """
    for module in _get_modules_in_order(module):
        if module.__name__ in sys.modules:
            try:
                sys.modules[module.__name__] = _reload(module)
            except ImportError:
                continue


def reload_all():
    """
    Reloads all imported modules in the correct order.
    """
    for name, module in sys.modules.items():
        if module and name != '__main__':
            reload_module(module)
